#include <stdlib.h>
#include "myLib.h"
#include "DMA.h"

u16 *videoBuffer = (u16 *) 0x6000000;

void setPixel3(int row, int col, u16 color) {
    
        videoBuffer[OFFSET(row, col, SCREENWIDTH)] = color;

}

void setPixel4(int row, int col, u8 index)
{
    int whichPixel = OFFSET(row, col, 240);
    int whichShort = whichPixel/2;
    if(col & 1)
    {
        // Odd Column case
        videoBuffer[whichShort] = (videoBuffer[whichShort] & 0x00FF) | (index<<8);
    }
    else
    {
        //Even column case
        videoBuffer[whichShort] = (videoBuffer[whichShort] & 0xFF00) | (index);
    }
}

void drawRect(int row, int col, int height, int width, u8 index) {
    
    int r;
    volatile u16 color = (index<<8) | index;
    for (r = 0; r < height; r++) {
        DMA[3].src = &color;
        DMA[3].dst = videoBuffer + OFFSET(r+row, col, SCREENWIDTH )/2;
        DMA[3].cnt = (width/2) | DMA_ON | DMA_SOURCE_FIXED;
    }

} 

void drawHorizLine(int row, int col, int length, u8 color) {

    int colfinal, i;

    if (length > 0) {
        colfinal = col + length;
        for (i = col; i < colfinal; i++) {
            setPixel4(row, i, color);
        }
    } else if (length < 0) {
        colfinal = col + length;
        for (i = colfinal; i < col; i++) {
            setPixel4(row, i, color);
        }
    }

}

void drawVertLine(int row, int col, int length, u8 color) {

    int rowfinal, i;
    if (length > 0) {
        rowfinal = row + length;
        for (i = row; i < rowfinal; i++) {
            setPixel4(i, col, color);
        }
    } else if (length < 0) {
        rowfinal = row + length;
        for (i = rowfinal; i < row; i++) {
            setPixel4(i, col, color);
        }
    }

}

void drawHollowRect(int row, int col, int height, int width, int thickness, u8 color) {
    int r = row;
    for (r = r; r < row + thickness; r++) {
        drawHorizLine(r, col, width, color);
    }
    int r1 = row + height - thickness;
    for (r1 = r1; r1 < row + height; r1++) {
        drawHorizLine(r1, col, width, color);
    }
    int c = col;
    c = c;
    for (c = c; c < col + thickness; c++) {
        drawVertLine(row + thickness, c, height - (thickness * 2), color);
    }
    int c1 = col + width - thickness;
    for (c1 = c1; c1 < col + width; c1++) {
        drawVertLine(row + thickness, c1, height - (thickness * 2), color);
    }
}

void rasterCircle(int row, int col, int radius, u8 color) {

    int f = 1 - radius;
    int ddF_x = 1;
    int ddF_y = -2 * radius;
    int x = 0;
    int y = radius;

    setPixel4(row, col + radius, color);
    setPixel4(row, col - radius, color);
    setPixel4(row + radius, col, color);
    setPixel4(row - radius, col, color);


    while (x < y) {
        if (f >= 0) {
            y--;
            ddF_y += 2;
            f += ddF_y;
        }
        x++;
        ddF_x += 2;
        f += ddF_x;
        setPixel4(row + x, col + y, color);
        setPixel4(row - x, col + y, color);
        setPixel4(row + x, col - y, color);
        setPixel4(row - x, col - y, color);
        setPixel4(row + y, col + x, color);
        setPixel4(row - y, col + x, color);
        setPixel4(row + y, col - x, color);
        setPixel4(row - y, col - x, color);
    }

}

void rasterCircleFace(int row, int col, int radius, u8 color) {

    int f = 1 - radius;
    int ddF_x = 1;
    int ddF_y = -2 * radius;
    int x = 0;
    int y = radius;

    setPixel4(row, col + radius, color);
    setPixel4(row, col - radius, color);
    setPixel4(row + radius, col, color);
    setPixel4(row - radius, col, color);

    while (x < y) {
        if (f >= 0) {
            y--;
            ddF_y += 2;
            f += ddF_y;
        }
        x++;
        ddF_x += 2;
        f += ddF_x;
        setPixel4(row + x, col + y, color);
        setPixel4(row - x, col + y, color);
        setPixel4(row + x, col - y, color);
        setPixel4(row - x, col - y, color);
        setPixel4(row + y, col + x, color);
        setPixel4(row - y, col + x, color);
        setPixel4(row + y, col - x, color);
        setPixel4(row - y, col - x, color);
        setPixel4(row - (y / 2), col - (y / 2), color);
        setPixel4(row - (y / 2), col + (y / 2), color);
        drawVertLine(row - (radius / 5), col, (radius / 3), color);
        drawHollowRect(row + (y / 2), col - (y / 5), (radius / 3), (radius / 3), 1, color);

    }
}

void drawStar(int row, int col, int rc, u8 color)
{
    int i;
    if(rc)
    {
        for(i=0; i<9;i++)
        {
            setPixel4(row+4, col+i, changeColor());
            setPixel4(row+i, col+4, changeColor());
            setPixel4(row+i, col+i, changeColor());
            setPixel4(row+i, col+8-i, changeColor());
        }
    }
    else
    {
       for(i=0; i<9;i++)
        {
            setPixel4(row+4, col+i, color);
            setPixel4(row+i, col+4, color);
            setPixel4(row+i, col+i, color);
            setPixel4(row+i, col+8-i, color);
        }
    }
}

void grid(){

    waitForVblank();    
    drawRect(78, 0, 4, 75, WHITEIDX);

    waitForVblank();    
    drawRect(0, 112, 122, 4, WHITEIDX);

    waitForVblank();    
    drawRect(118, 116, 4, 90, WHITEIDX);

    waitForVblank();    
    drawRect(78, 152, 4, 87, WHITEIDX);

}

u16 changeColor() {

    int random = rand()*32;

    return COLOR(random, random, random);

}

void clrscr(){

    drawRect(0, 0, 160, 240, BLACKIDX);

};

void waitForVblank() {
    while (SCANLINECOUNTER > 160);
    while (SCANLINECOUNTER < 160);
}

void pause() {
    //volatile int x=0;
    volatile int delay;
     for (delay = 0; delay < 2000; delay++) {
       //  x++;//waste time
     }
}

void FlipPage()
{
    if(REG_DISPCTL & BUFFER1FLAG)
    {
        // We were displaying Buffer 1 now display buffer 0
        REG_DISPCTL = REG_DISPCTL & ~BUFFER1FLAG;
        videoBuffer = BUFFER1;
    }
    else
    {
        REG_DISPCTL = REG_DISPCTL | BUFFER1FLAG;
        videoBuffer = BUFFER0;
    }
}